@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------
-# posologyr: individual dose optimisation using population PK
+# posologyr: individual dose optimization using population PK
 # Copyright (C) Cyril Leven
 #
 #    This program is free software: you can redistribute it and/or modify
@@ -16,10 +16,10 @@
 # along with this program. If not, see <https://www.gnu.org/licenses/>.
 #-------------------------------------------------------------------------
 
-#' Predict time to a selected trough concentration
+#' Estimate the time required to reach a target trough concentration (Cmin)
 #'
-#' Predicts the time needed to reach a selected trough concentration
-#' (Cmin) given a population pharmacokinetic model, a set of individual
+#' Estimates the time required to reach a target trough concentration (Cmin)
+#' given a population pharmacokinetic model, a set of individual
 #' parameters, a dose, and a target Cmin.
 #'
 #' @param dat Dataframe. An individual subject dataset following the
@@ -42,6 +42,9 @@
 #' @param target_cmin Numeric. Target trough concentration (Cmin).
 #' @param dose Numeric. Dose administered. This argument is ignored if `tdm` is
 #'    set to `TRUE`.
+#' @param cmt_dose Character or numeric. The compartment in which the dose is
+#'    to be administered. Must match one of the compartments in the prior model.
+#'    Defaults to 1.
 #' @param endpoint Character. The endpoint of the prior model to be optimised
 #'    for. The default is "Cc", which is the central concentration.
 #' @param estim_method A character string. An estimation method to be used for
@@ -97,37 +100,33 @@
 #' rxode2::setRxThreads(2L) # limit the number of threads
 #'
 #' # model
-#' mod_run001 <- list(
-#' ppk_model = rxode2::rxode({
-#'   centr(0) = 0;
-#'   depot(0) = 0;
+#' mod_run001 <- function() {
+#'   ini({
+#'     THETA_Cl <- 4.0
+#'     THETA_Vc <- 70.0
+#'     THETA_Ka <- 1.0
+#'     ETA_Cl ~ 0.2
+#'     ETA_Vc ~ 0.2
+#'     ETA_Ka ~ 0.2
+#'     prop.sd <- sqrt(0.05)
+#'   })
+#'   model({
+#'     TVCl <- THETA_Cl
+#'     TVVc <- THETA_Vc
+#'     TVKa <- THETA_Ka
 #'
-#'   TVCl = THETA_Cl;
-#'   TVVc = THETA_Vc;
-#'   TVKa = THETA_Ka;
+#'     Cl <- TVCl*exp(ETA_Cl)
+#'     Vc <- TVVc*exp(ETA_Vc)
+#'     Ka <- TVKa*exp(ETA_Ka)
 #'
-#'   Cl = TVCl*exp(ETA_Cl);
-#'   Vc = TVVc*exp(ETA_Vc);
-#'   Ka = TVKa*exp(ETA_Ka);
+#'     K20 <- Cl/Vc
+#'     Cc <- centr/Vc
 #'
-#'   K20 = Cl/Vc;
-#'   Cc = centr/Vc;
-#'
-#'   d/dt(depot) = -Ka*depot;
-#'   d/dt(centr) = Ka*depot - K20*centr;
-#'   d/dt(AUC) = Cc;
-#' }),
-#' error_model = function(f,sigma) {
-#'   dv <- cbind(f,1)
-#'   g <- diag(dv%*%sigma%*%t(dv))
-#'   return(sqrt(g))
-#' },
-#' theta = c(THETA_Cl=4.0, THETA_Vc=70.0, THETA_Ka=1.0),
-#' omega = lotri::lotri({ETA_Cl + ETA_Vc + ETA_Ka ~
-#'     c(0.2,
-#'       0, 0.2,
-#'       0, 0, 0.2)}),
-#' sigma = lotri::lotri({prop + add ~ c(0.05,0.0,0.00)}))
+#'     d/dt(depot) = -Ka*depot
+#'     d/dt(centr) = Ka*depot - K20*centr
+#'     Cc ~ prop(prop.sd)
+#'   })
+#' }
 #' # df_patient01: event table for Patient01, following a 30 minutes intravenous
 #' # infusion
 #' df_patient01 <- data.frame(ID=1,
@@ -144,7 +143,7 @@
 #'
 #' @export
 poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
-                           target_cmin,dose=NULL,endpoint="Cc",
+                           target_cmin,dose=NULL,cmt_dose=1,endpoint="Cc",
                            estim_method="map",nocb=FALSE,p=NULL,
                            greater_than=TRUE,from=0.2,last_time=72,
                            add_dose=NULL,interdose_interval=NULL,
@@ -232,7 +231,7 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
     #simulation of the individual scenario
     # create an event table with the required number of administrations
     if (!is.null(add_dose)){ #more than one dose is needed
-      event_table_cmin <- rxode2::et(amt=dose,dur=duration,
+      event_table_cmin <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose,
                                      ii=interdose_interval,
                                      addl=add_dose)
       time_last_dose   <- add_dose*interdose_interval
@@ -242,7 +241,7 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
                                         by=0.1))
     }
     else { #only one dose is needed: simulation of a single administration
-      event_table_cmin <- rxode2::et(amt=dose,dur=duration)
+      event_table_cmin <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose)
       event_table_cmin$add.sampling(seq(from,last_time,by=0.1))
       time_last_dose   <- 0
     }
@@ -315,12 +314,12 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
   return(time_cmin)
 }
 
-#' Estimate the optimal dose for a selected target area under the
-#' time-concentration curve (AUC)
+#' Estimate the dose needed to reach a target area under the concentration-time
+#' curve (AUC)
 #'
-#' Estimates the optimal dose for a selected target area under the
-#' time-concentration curve (AUC) given a population pharmacokinetic
-#' model, a set of individual parameters, and a target AUC.
+#' estimates the dose needed to reach a target area under the concentration-time
+#' curve (AUC) given a population pharmacokinetic model, a set of individual
+#' parameters, and a target AUC.
 #'
 #' @param dat Dataframe. An individual subject dataset following the
 #'    structure of NONMEM/rxode2 event records.
@@ -342,6 +341,9 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'    `time_dose` + `time_auc` instead.
 #' @param time_dose Numeric. Time when the dose is to be given. Only used and
 #'    mandatory, when `tdm` is set to `TRUE`.
+#' @param cmt_dose Character or numeric. The compartment in which the dose is
+#'    to be administered. Must match one of the compartments in the prior model.
+#'    Defaults to 1.
 #' @param target_auc Numeric. The target AUC.
 #' @param estim_method A character string. An estimation method to be used for
 #'    the individual parameters. The default method "map" is the Maximum A
@@ -393,37 +395,33 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #' rxode2::setRxThreads(2L) # limit the number of threads
 #'
 #' # model
-#' mod_run001 <- list(
-#' ppk_model = rxode2::rxode({
-#'   centr(0) = 0;
-#'   depot(0) = 0;
+#' mod_run001 <- function() {
+#'   ini({
+#'     THETA_Cl <- 4.0
+#'     THETA_Vc <- 70.0
+#'     THETA_Ka <- 1.0
+#'     ETA_Cl ~ 0.2
+#'     ETA_Vc ~ 0.2
+#'     ETA_Ka ~ 0.2
+#'     prop.sd <- sqrt(0.05)
+#'   })
+#'   model({
+#'     TVCl <- THETA_Cl
+#'     TVVc <- THETA_Vc
+#'     TVKa <- THETA_Ka
 #'
-#'   TVCl = THETA_Cl;
-#'   TVVc = THETA_Vc;
-#'   TVKa = THETA_Ka;
+#'     Cl <- TVCl*exp(ETA_Cl)
+#'     Vc <- TVVc*exp(ETA_Vc)
+#'     Ka <- TVKa*exp(ETA_Ka)
 #'
-#'   Cl = TVCl*exp(ETA_Cl);
-#'   Vc = TVVc*exp(ETA_Vc);
-#'   Ka = TVKa*exp(ETA_Ka);
+#'     K20 <- Cl/Vc
+#'     Cc <- centr/Vc
 #'
-#'   K20 = Cl/Vc;
-#'   Cc = centr/Vc;
-#'
-#'   d/dt(depot) = -Ka*depot;
-#'   d/dt(centr) = Ka*depot - K20*centr;
-#'   d/dt(AUC) = Cc;
-#' }),
-#' error_model = function(f,sigma) {
-#'   dv <- cbind(f,1)
-#'   g <- diag(dv%*%sigma%*%t(dv))
-#'   return(sqrt(g))
-#' },
-#' theta = c(THETA_Cl=4.0, THETA_Vc=70.0, THETA_Ka=1.0),
-#' omega = lotri::lotri({ETA_Cl + ETA_Vc + ETA_Ka ~
-#'     c(0.2,
-#'       0, 0.2,
-#'       0, 0, 0.2)}),
-#' sigma = lotri::lotri({prop + add ~ c(0.05,0.0,0.00)}))
+#'     d/dt(depot) = -Ka*depot
+#'     d/dt(centr) = Ka*depot - K20*centr
+#'     Cc ~ prop(prop.sd)
+#'   })
+#' }
 #' # df_patient01: event table for Patient01, following a 30 minutes intravenous
 #' # infusion
 #' df_patient01 <- data.frame(ID=1,
@@ -438,7 +436,7 @@ poso_time_cmin <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'
 #' @export
 poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
-                          time_auc,time_dose=NULL,target_auc,
+                          time_auc,time_dose=NULL,cmt_dose=1,target_auc,
                           estim_method="map",nocb=FALSE,
                           p=NULL,greater_than=TRUE,starting_time=0,
                           interdose_interval=NULL,add_dose=NULL,
@@ -513,10 +511,10 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
       #New dose at time_dose in the extended_et
       # Shorter syntax with "list" allows for setting several variable at once
       extended_et[dose_and_start[1],
-                  c("evid","amt","dur"):=list(1,dose,duration)]
+                  c("evid","amt","cmt","dur"):=list(1,dose,cmt_dose,duration)]
       #New observation at time_dose and ending_time
-      extended_et[dose_and_start[2],c("evid","amt","dur"):=list(0,NA,NA)]
-      extended_et[time==ending_time,c("evid","amt","dur"):=list(0,NA,NA)]
+      extended_et[dose_and_start[2],c("evid","amt","cmt","dur"):=list(0,NA,NA,NA)]
+      extended_et[time==ending_time,c("evid","amt","cmt","dur"):=list(0,NA,NA,NA)]
       #Solve the model with the extended et
       auc_ppk_model <- rxode2::rxSolve(prior_model$ppk_model,extended_et,
                                          c(prior_model$theta,auc_map$eta),
@@ -568,11 +566,11 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 
       #compute the individual time-concentration profile
       if (!is.null(add_dose)){
-        event_table_auc <- rxode2::et(amt=dose,dur=duration,
+        event_table_auc <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose,
                                       ii=interdose_interval,
                                       addl=add_dose)
       } else {
-        event_table_auc <- rxode2::et(amt=dose,dur=duration)
+        event_table_auc <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose)
       }
 
       event_table_auc$add.sampling(starting_time)
@@ -642,12 +640,11 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
   return(dose_auc)
 }
 
-#' Estimate the optimal dose for a selected target concentration
+#' Estimate the optimal dose to achieve a target concentration at any given time
 #'
-#' Estimates the optimal dose for a selected target concentration at a
-#' selected point in time given a population pharmacokinetic model, a set
-#' of individual parameters, a selected point in time, and a target
-#' concentration.
+#' Estimates the optimal dose to achieve a target concentration at any given
+#' time given a population pharmacokinetic model, a set of individual
+#' parameters, a selected point in time, and a target concentration.
 #'
 #' @param dat Dataframe. An individual subject dataset following the
 #'     structure of NONMEM/rxode2 event records.
@@ -665,6 +662,9 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'     optimized.
 #' @param time_dose Numeric. Time when the dose is to be given.
 #' @param target_conc Numeric. Target concentration.
+#' @param cmt_dose Character or numeric. The compartment in which the dose is
+#'    to be administered. Must match one of the compartments in the prior model.
+#'    Defaults to 1.
 #' @param endpoint Character. The endpoint of the prior model to be optimised
 #'    for. The default is "Cc", which is the central concentration.
 #' @param estim_method A character string. An estimation method to be used for
@@ -716,37 +716,33 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #' rxode2::setRxThreads(2L) # limit the number of threads
 #'
 #' # model
-#' mod_run001 <- list(
-#' ppk_model = rxode2::rxode({
-#'   centr(0) = 0;
-#'   depot(0) = 0;
+#' mod_run001 <- function() {
+#'   ini({
+#'     THETA_Cl <- 4.0
+#'     THETA_Vc <- 70.0
+#'     THETA_Ka <- 1.0
+#'     ETA_Cl ~ 0.2
+#'     ETA_Vc ~ 0.2
+#'     ETA_Ka ~ 0.2
+#'     prop.sd <- sqrt(0.05)
+#'   })
+#'   model({
+#'     TVCl <- THETA_Cl
+#'     TVVc <- THETA_Vc
+#'     TVKa <- THETA_Ka
 #'
-#'   TVCl = THETA_Cl;
-#'   TVVc = THETA_Vc;
-#'   TVKa = THETA_Ka;
+#'     Cl <- TVCl*exp(ETA_Cl)
+#'     Vc <- TVVc*exp(ETA_Vc)
+#'     Ka <- TVKa*exp(ETA_Ka)
 #'
-#'   Cl = TVCl*exp(ETA_Cl);
-#'   Vc = TVVc*exp(ETA_Vc);
-#'   Ka = TVKa*exp(ETA_Ka);
+#'     K20 <- Cl/Vc
+#'     Cc <- centr/Vc
 #'
-#'   K20 = Cl/Vc;
-#'   Cc = centr/Vc;
-#'
-#'   d/dt(depot) = -Ka*depot;
-#'   d/dt(centr) = Ka*depot - K20*centr;
-#'   d/dt(AUC) = Cc;
-#' }),
-#' error_model = function(f,sigma) {
-#'   dv <- cbind(f,1)
-#'   g <- diag(dv%*%sigma%*%t(dv))
-#'   return(sqrt(g))
-#' },
-#' theta = c(THETA_Cl=4.0, THETA_Vc=70.0, THETA_Ka=1.0),
-#' omega = lotri::lotri({ETA_Cl + ETA_Vc + ETA_Ka ~
-#'     c(0.2,
-#'       0, 0.2,
-#'       0, 0, 0.2)}),
-#' sigma = lotri::lotri({prop + add ~ c(0.05,0.0,0.00)}))
+#'     d/dt(depot) = -Ka*depot
+#'     d/dt(centr) = Ka*depot - K20*centr
+#'     Cc ~ prop(prop.sd)
+#'   })
+#' }
 #' # df_patient01: event table for Patient01, following a 30 minutes intravenous
 #' # infusion
 #' df_patient01 <- data.frame(ID=1,
@@ -762,8 +758,8 @@ poso_dose_auc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'
 #' @export
 poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
-                           time_c,time_dose=NULL,target_conc,endpoint="Cc",
-                           estim_method="map",nocb=FALSE,p=NULL,
+                           time_c,time_dose=NULL,target_conc,cmt_dose=1,
+                           endpoint="Cc",estim_method="map",nocb=FALSE,p=NULL,
                            greater_than=TRUE,starting_dose=100,
                            interdose_interval=NULL,add_dose=NULL,duration=0,
                            indiv_param=NULL){
@@ -828,9 +824,10 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
                              extended_et,nocb){
       #New dose at time_dose in the extended_et
       # Shorter syntax with "list" allows for setting several variable at once
-      extended_et[time==time_dose,c("evid","amt","dur"):=list(1,dose,duration)]
+      extended_et[time==time_dose,
+                  c("evid","amt","cmt","dur"):=list(1,dose,cmt_dose,duration)]
       #New observation at time_c
-      extended_et[time==time_c,c("evid","amt","dur"):=list(0,NA,NA)]
+      extended_et[time==time_c,c("evid","amt","cmt","dur"):=list(0,NA,NA,NA)]
       #Solve the model with the extended et
       ctime_ppk_model <- rxode2::rxSolve(prior_model$ppk_model,extended_et,
                                          c(prior_model$theta,ctime_map$eta),
@@ -880,12 +877,12 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 
       #compute the individual time-concentration profile
       if (!is.null(add_dose)){
-        event_table_ctime <- rxode2::et(amt=dose,dur=duration,
+        event_table_ctime <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose,
                                         ii=interdose_interval,
                                         addl=add_dose)
       }
       else {
-        event_table_ctime <- rxode2::et(amt=dose,dur=duration)
+        event_table_ctime <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose)
       }
       event_table_ctime$add.sampling(time_c)
 
@@ -944,13 +941,12 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
   return(dose_conc)
 }
 
-#' Estimate the optimal inter-dose interval for a given dose and a
-#' selected target trough concentration
+#' Estimate the optimal dosing interval to consistently achieve a target trough
+#' concentration (Cmin)
 #'
-#' Estimates the optimal inter-dose interval for a selected target
-#' trough concentration (Cmin), given a dose, a population
-#' pharmacokinetic model, a set of individual parameters, and a
-#' target concentration.
+#' Estimates the optimal dosing interval to consistently achieve a target Cmin,
+#' given a dose, a population pharmacokinetic model, a set of individual
+#' parameters, and a target concentration.
 #'
 #' @param dat Dataframe. An individual subject dataset following the
 #'     structure of NONMEM/rxode2 event records.
@@ -958,6 +954,9 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'    model, a list of six objects.
 #' @param target_cmin Numeric. Target trough concentration (Cmin).
 #' @param dose Numeric. The dose given.
+#' @param cmt_dose Character or numeric. The compartment in which the dose is
+#'    to be administered. Must match one of the compartments in the prior model.
+#'    Defaults to 1.
 #' @param endpoint Character. The endpoint of the prior model to be optimised
 #'    for. The default is "Cc", which is the central concentration.
 #' @param estim_method A character string. An estimation method to be used for
@@ -1002,37 +1001,33 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #' rxode2::setRxThreads(2L) # limit the number of threads
 #'
 #' # model
-#' mod_run001 <- list(
-#' ppk_model = rxode2::rxode({
-#'   centr(0) = 0;
-#'   depot(0) = 0;
+#' mod_run001 <- function() {
+#'   ini({
+#'     THETA_Cl <- 4.0
+#'     THETA_Vc <- 70.0
+#'     THETA_Ka <- 1.0
+#'     ETA_Cl ~ 0.2
+#'     ETA_Vc ~ 0.2
+#'     ETA_Ka ~ 0.2
+#'     prop.sd <- sqrt(0.05)
+#'   })
+#'   model({
+#'     TVCl <- THETA_Cl
+#'     TVVc <- THETA_Vc
+#'     TVKa <- THETA_Ka
 #'
-#'   TVCl = THETA_Cl;
-#'   TVVc = THETA_Vc;
-#'   TVKa = THETA_Ka;
+#'     Cl <- TVCl*exp(ETA_Cl)
+#'     Vc <- TVVc*exp(ETA_Vc)
+#'     Ka <- TVKa*exp(ETA_Ka)
 #'
-#'   Cl = TVCl*exp(ETA_Cl);
-#'   Vc = TVVc*exp(ETA_Vc);
-#'   Ka = TVKa*exp(ETA_Ka);
+#'     K20 <- Cl/Vc
+#'     Cc <- centr/Vc
 #'
-#'   K20 = Cl/Vc;
-#'   Cc = centr/Vc;
-#'
-#'   d/dt(depot) = -Ka*depot;
-#'   d/dt(centr) = Ka*depot - K20*centr;
-#'   d/dt(AUC) = Cc;
-#' }),
-#' error_model = function(f,sigma) {
-#'   dv <- cbind(f,1)
-#'   g <- diag(dv%*%sigma%*%t(dv))
-#'   return(sqrt(g))
-#' },
-#' theta = c(THETA_Cl=4.0, THETA_Vc=70.0, THETA_Ka=1.0),
-#' omega = lotri::lotri({ETA_Cl + ETA_Vc + ETA_Ka ~
-#'     c(0.2,
-#'       0, 0.2,
-#'       0, 0, 0.2)}),
-#' sigma = lotri::lotri({prop + add ~ c(0.05,0.0,0.00)}))
+#'     d/dt(depot) = -Ka*depot
+#'     d/dt(centr) = Ka*depot - K20*centr
+#'     Cc ~ prop(prop.sd)
+#'   })
+#' }
 #' # df_patient01: event table for Patient01, following a 30 minutes intravenous
 #' # infusion
 #' df_patient01 <- data.frame(ID=1,
@@ -1048,9 +1043,10 @@ poso_dose_conc <- function(dat=NULL,prior_model=NULL,tdm=FALSE,
 #'
 #' @export
 poso_inter_cmin <- function(dat=NULL,prior_model=NULL,dose,target_cmin,
-                            endpoint="Cc",estim_method="map",nocb=FALSE,p=NULL,
-                            greater_than=TRUE,starting_interval=12,add_dose=10,
-                            duration=0,indiv_param=NULL){
+                            cmt_dose=1,endpoint="Cc",estim_method="map",
+                            nocb=FALSE,p=NULL,greater_than=TRUE,
+                            starting_interval=12,add_dose=10,duration=0,
+                            indiv_param=NULL){
 
   prior_model <- get_prior_model(prior_model)
 
@@ -1071,7 +1067,7 @@ poso_inter_cmin <- function(dat=NULL,prior_model=NULL,dose,target_cmin,
                         prior_model,add_dose,duration=duration,
                         indiv_param){
     #compute the individual time-concentration profile
-    event_table_cmin <- rxode2::et(amt=dose,dur=duration,
+    event_table_cmin <- rxode2::et(amt=dose,dur=duration,cmt=cmt_dose,
                                   ii=interdose_interval,
                                   addl=add_dose)
     event_table_cmin$add.sampling(interdose_interval*add_dose-0.1)
